@@ -25,8 +25,8 @@ static const char* contrast_curve_cl_source =
 "  int gid     = get_global_id(0);                                             \n"
 "  float2 in_v = in[gid];                                                      \n"
 "                                                                              \n"
-"  int idx = (int) fmin(num_sampling_points - 1,                               \n"
-"                       fmax(0f,                                               \n"
+"  int idx = (int) fmin(num_sampling_points - 1.0f,                            \n"
+"                       fmax(0.0f,                                             \n"
 "                            in_v.x * num_sampling_points));                   \n"
 "                                                                              \n"
 "  out[gid] = (float2) (curve[idx], in_v.y);                                   \n"
