@@ -13,11 +13,12 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright 2014 Øyvind Kolås <pippin@gimp.org>
+ * Copyright 2014, 2018 Øyvind Kolås <pippin@gimp.org>
  */
 
 #include <math.h>
 
+
 #ifdef GEGL_PROPERTIES
 
 property_double (pan, _("Pan"), 0.0)
@@ -51,12 +52,15 @@ property_int    (height, _("Height"), -1)
   ui_meta       ("role", "output-extent")
   ui_meta       ("axis", "y")
 
+property_boolean(inverse, _("Inverse transform"), FALSE)
+  description   (_("Do the inverse mapping, useful for touching up zenith, nadir or other parts of panorama."))
+
 property_boolean(little_planet, _("Little planet"), FALSE)
   description   (_("Render a stereographic mapping, a tilt value of 90, which means looking at nadir provides a good default value."))
 
 property_enum   (sampler_type, _("Resampling method"),
                   GeglSamplerType, gegl_sampler_type, GEGL_SAMPLER_NEAREST)
-  description   (_("Image resampling method to use"))
+  description   (_("Image resampling method to use, for good results with double resampling when retouching panoramas, use nearest to generate the view and cubic or better for the inverse transform back to panorama."))
 
 #else
 
@@ -84,8 +88,10 @@ struct _Transform
   float xoffset;
   float width;
   float height;
-  void (*xy2ll) (Transform *transform, float x, float  y, float *lon, float *lat);
-  void (*ll2xy) (Transform *transform, float lon, float  lat, float *x, float *y);
+  float in_width;
+  float in_height;
+  void (*mapfun) (Transform *transform, float x, float  y, float *lon, float *lat);
+  int   reverse;
   int   do_spin;
   int   do_zoom;
 };
@@ -101,6 +107,9 @@ gnomonic_xy2ll (Transform *transform, float x, float y,
   float longtitude, latitude;
   float sin_c, cos_c;
 
+  y -= 0.5f;
+  x -= transform->xoffset;
+
   if (transform->do_spin)
   {
     float tx = x, ty = y;
@@ -123,7 +132,7 @@ gnomonic_xy2ll (Transform *transform, float x, float y,
   latitude = asinf (cos_c * transform->sin_tilt + ( y * sin_c * transform->cos_tilt) / p);
   longtitude = transform->pan + atan2f (x * sin_c, p * transform->cos_tilt * cos_c - y * transform->sin_tilt * sin_c);
 
-  if (longtitude < 0)
+  if (longtitude < 0.0f)
     longtitude += M_PI * 2;
 
   *lon = (longtitude / (M_PI * 2));
@@ -135,18 +144,29 @@ gnomonic_ll2xy (Transform *transform,
                 float lon, float lat,
                 float *x, float *y)
 {
-  float cos_c;
-  float sin_lon = sinf (lon);
-  float cos_lon = cosf (lon);
-  float cos_lat_minus_pan = cosf (lat - transform->pan);
+  float cos_c, sin_lat, cos_lat, cos_lon_minus_pan;
 
-  cos_c = (transform->sin_tilt * sin_lon +
-           transform->cos_tilt * cos (lon) *
-            cos_lat_minus_pan);
+  lat = lat * M_PI - M_PI/2;
+  lon = lon * (M_PI * 2);
 
-  *x = ((cos_lon * sin (lat - transform->pan)) / cos_c);
-  *y = ((transform->cos_tilt * sin_lon -
-         transform->sin_tilt * cos_lon * cos_lat_minus_pan) / cos_c);
+  sin_lat = sinf (lat);
+  cos_lat = cosf (lat);
+  cos_lon_minus_pan = cosf (lon - transform->pan);
+
+  cos_c = (transform->sin_tilt * sin_lat +
+           transform->cos_tilt * cos_lat *
+            cos_lon_minus_pan);
+
+  if (cos_c <= 0.01f)
+  {
+    *x = -.1f;
+    *y = -.1f;
+    return;
+  }
+
+  *x = ((cos_lat * sin (lon - transform->pan)) / cos_c);
+  *y = ((transform->cos_tilt * sin_lat -
+         transform->sin_tilt * cos_lat * cos_lon_minus_pan) / cos_c);
 
   if (transform->do_zoom)
   {
@@ -159,6 +179,10 @@ gnomonic_ll2xy (Transform *transform,
     *x = tx * transform->cos_negspin - ty * transform->sin_negspin;
     *y = ty * transform->cos_negspin + tx * transform->sin_negspin;
   }
+
+  *x += transform->xoffset;
+  *y += 0.5f;
+
 }
 
 static void inline
@@ -166,18 +190,22 @@ stereographic_ll2xy (Transform *transform,
                      float lon, float lat,
                      float *x,  float *y)
 {
-  float k;
-  float sin_lon = sinf (lon);
-  float cos_lon = cosf (lon);
-  float cos_lat_minus_pan = cosf (lat - transform->pan);
+  float k, sin_lat, cos_lat, cos_lon_minus_pan;
 
-  k = 2/(1 + transform->sin_tilt * sin_lon +
-            transform->cos_tilt * cos (lon) *
-            cos_lat_minus_pan);
+  lat = lat * M_PI - M_PI/2;
+  lon = lon * (M_PI * 2);
 
-  *x = k * ((cos_lon * sin (lat - transform->pan)));
-  *y = k * ((transform->cos_tilt * sin_lon -
-        transform->sin_tilt * cos_lon * cos_lat_minus_pan));
+  sin_lat = sinf (lat);
+  cos_lat = cosf (lat);
+  cos_lon_minus_pan = cosf (lon - transform->pan);
+
+  k = 2/(1 + transform->sin_tilt * sin_lat +
+             transform->cos_tilt * cos_lat *
+             cos_lon_minus_pan);
+
+  *x = k * ((cos_lat * sin (lon - transform->pan)));
+  *y = k * ((transform->cos_tilt * sin_lat -
+        transform->sin_tilt * cos_lat * cos_lon_minus_pan));
 
   if (transform->do_zoom)
   {
@@ -191,6 +219,9 @@ stereographic_ll2xy (Transform *transform,
     *x = tx * transform->cos_negspin - ty * transform->sin_negspin;
     *y = ty * transform->cos_negspin + tx * transform->sin_negspin;
   }
+
+  *x += transform->xoffset;
+  *y += 0.5f;
 }
 
 static void inline
@@ -202,6 +233,9 @@ stereographic_xy2ll (Transform *transform,
   float longtitude, latitude;
   float sin_c, cos_c;
 
+  y -= 0.5f;
+  x -= transform->xoffset;
+
   if (transform->do_spin)
   {
     float tx = x, ty = y;
@@ -235,15 +269,29 @@ static void prepare_transform (Transform *transform,
                                float pan, float spin, float zoom, float tilt,
                                int little_planet,
                                float width, float height,
-                               float input_width, float input_height)
+                               float input_width, float input_height,
+                               int inverse)
 {
-  float xoffset = 0.5;
-  transform->xy2ll = gnomonic_xy2ll;
-  transform->ll2xy = gnomonic_ll2xy;
+  float xoffset = 0.5f;
+  transform->reverse = inverse;
+  if (little_planet)
+  {
+    if (inverse)
+      transform->mapfun = stereographic_ll2xy;
+    else
+      transform->mapfun = stereographic_xy2ll;
+  }
+  else
+  {
+    if (inverse)
+      transform->mapfun = gnomonic_ll2xy;
+    else
+      transform->mapfun = gnomonic_xy2ll;
+  }
 
   pan  = pan / 360 * M_PI * 2;
   spin = spin / 360 * M_PI * 2;
-  zoom = little_planet?zoom / 1000.0:zoom / 100.0;
+  zoom = little_planet?zoom / 1000.0f:zoom / 100.0f;
   tilt = tilt / 360 * M_PI * 2;
 
   while (pan > M_PI)
@@ -253,20 +301,16 @@ static void prepare_transform (Transform *transform,
   {
     width = input_height;
     height = width;
-    xoffset = ((input_width - height)/height) / 2 + 0.5;
+    xoffset = ((input_width - height)/height) / 2 + 0.5f;
   }
   else
   {
     float orig_width = width;
     width = height;
-    xoffset = ((orig_width - height)/height) / 2 + 0.5;
+    xoffset = ((orig_width - height)/height) / 2 + 0.5f;
   }
 
-  if (little_planet)
-  {
-    transform->xy2ll = stereographic_xy2ll;
-    transform->ll2xy = stereographic_ll2xy;
-  }
+
 
   transform->do_spin = fabs (spin) > 0.000001 ? 1 : 0;
   transform->do_zoom = fabs (zoom-1.0) > 0.000001 ? 1 : 0;
@@ -284,12 +328,28 @@ static void prepare_transform (Transform *transform,
   transform->cos_negspin = cosf (-spin);
   transform->width       = width;
   transform->height      = height;
+  transform->in_width    = input_width;
+  transform->in_height   = input_height;
+
+  if (inverse)
+  {
+    float tmp;
+#define swap(a,b) tmp = a; a = b; b= tmp;
+    swap(transform->width, transform->in_width);
+    swap(transform->height, transform->in_height);
+#undef swap
+  }
 }
 
 static void
 prepare (GeglOperation *operation)
 {
-  const Babl *format = babl_format ("RaGaBaA float");
+  GeglProperties *o = GEGL_PROPERTIES (operation);
+  const Babl *format;
+  if (o->sampler_type == GEGL_SAMPLER_NEAREST)
+    format = babl_format ("RGBA float");
+  else
+    format = babl_format ("RaGaBaA float");
 
   gegl_operation_set_format (operation, "input", format);
   gegl_operation_set_format (operation, "output", format);
@@ -323,7 +383,6 @@ get_bounding_box (GeglOperation *operation)
   return result;
 }
 
-
 static void prepare_transform2 (Transform *transform,
                                 GeglOperation *operation,
                                 gint level)
@@ -335,7 +394,9 @@ static void prepare_transform2 (Transform *transform,
   prepare_transform (transform,
                      o->pan, o->spin, o->zoom, o->tilt,
                      o->little_planet, o->width / factor, o->height / factor,
-                     in_rect.width, in_rect.height);
+                     in_rect.width, in_rect.height,
+                     o->inverse);
+
 }
 
 static GeglRectangle
@@ -347,7 +408,6 @@ get_required_for_output (GeglOperation       *operation,
   return result;
 }
 
-
 static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *input,
@@ -357,14 +417,15 @@ process (GeglOperation       *operation,
 {
   GeglProperties *o = GEGL_PROPERTIES (operation);
   Transform           transform;
-  const Babl         *format_io;
   GeglSampler        *sampler;
-  gint                factor = 1 << level;
+  gint                factor       = 1 << level;
   GeglBufferIterator *it;
-  GeglRectangle in_rect = *gegl_operation_source_get_bounding_box (operation, "input");
-  GeglMatrix2  scale_matrix;
-  GeglMatrix2 *scale = NULL;
-  gint sampler_type = o->sampler_type;
+  GeglMatrix2         scale_matrix;
+  GeglMatrix2        *scale        = NULL;
+  gint                sampler_type = o->sampler_type;
+  const Babl         *format_io    = babl_format ("RaGaBaA float");
+  GeglSamplerGetFun   getfun;
+
   level = 0;
   factor = 1;
   prepare_transform2 (&transform, operation, level);
@@ -372,28 +433,33 @@ process (GeglOperation       *operation,
   if (level)
     sampler_type = GEGL_SAMPLER_NEAREST;
 
-  format_io = babl_format ("RaGaBaA float");
+  if (transform.reverse)
   {
-    /* XXX: panorama projection needs to sample from a higher resolution than
-     * its output to yield good results. This affects which level should
-     * be rendered for source nodes..
-     */
+    /* artifacts have been observed with these samplers */
+    if (sampler_type == GEGL_SAMPLER_NOHALO ||
+        sampler_type == GEGL_SAMPLER_LOHALO)
+      sampler_type = GEGL_SAMPLER_CUBIC;
 
-    gint sample_level = level - 3;
-
-    if (sample_level < 0)
-      sample_level = 0;
-    sampler = gegl_buffer_sampler_new_at_level (input, format_io, sampler_type,
-                                       sample_level);
+    if (o->little_planet)
+      sampler_type = GEGL_SAMPLER_NEAREST;
   }
 
-  if (sampler_type == GEGL_SAMPLER_NOHALO ||
-      sampler_type == GEGL_SAMPLER_LOHALO)
+  if (sampler_type != GEGL_SAMPLER_NEAREST &&
+      !(o->little_planet == FALSE && o->inverse == FALSE &&
+      abs(o->tilt < 33)))
+    /* skip the computation of sampler neighborhood scale matrix in cases where
+     * we are unlikely to be scaling down */
     scale = &scale_matrix;
+  else
+    format_io = babl_format ("RGBA float");
+
+  sampler = gegl_buffer_sampler_new_at_level (input, format_io, sampler_type, 0);
+  getfun = gegl_sampler_get_fun (sampler);
 
   {
-    float   ud = ((1.0/transform.width)*factor);
-    float   vd = ((1.0/transform.height)*factor);
+    float   ud = ((1.0f/transform.width)*factor);
+    float   vd = ((1.0f/transform.height)*factor);
+    int abyss_mode = transform.reverse ? GEGL_ABYSS_NONE : GEGL_ABYSS_LOOP;
 
     it = gegl_buffer_iterator_new (output, result, level, format_io,
                                    GEGL_ACCESS_WRITE, GEGL_ABYSS_NONE);
@@ -402,42 +468,76 @@ process (GeglOperation       *operation,
       {
         gint i;
         gint n_pixels = it->length;
-        gint x = it->roi->x; /* initial x                   */
-        gint y = it->roi->y; /*           and y coordinates */
+        gint x = it->roi->width; /* initial x                   */
 
-        float   u0 = (((x*factor)/transform.width) - transform.xoffset);
+        float   u0 = (((it->roi->x*factor * 1.0f)/transform.width));
         float   u, v;
 
         float *out = it->data[0];
 
         u = u0;
-        v = ((y*factor/transform.height) - 0.5);
+        v = ((it->roi->y*factor * 1.0/transform.height));
 
         if (scale)
           {
             for (i=0; i<n_pixels; i++)
               {
                 float cx, cy;
-#define gegl_unmap(xx,yy,ud,vd) {                                       \
-                  float rx, ry;                                         \
-                  transform.xy2ll (&transform, xx, yy, &rx, &ry);       \
+/* we need our own jacobian matrix approximator,
+ * since we do not operate on pixel values
+ */
+#define gegl_sampler_compute_scale2(matrix, x, y)        \
+{                                                        \
+  float ax, ay, bx, by;                                  \
+  gegl_unmap(x + 0.5 * ud, y, ax, ay);                   \
+  gegl_unmap(x - 0.5 * ud, y, bx, by);                   \
+  matrix.coeff[0][0] = (ax - bx);   \
+  matrix.coeff[1][0] = (ay - by);  \
+  gegl_unmap(x, y + 0.5 * ud, ax, ay);                   \
+  gegl_unmap(x, y - 0.5 * ud, bx, by);                   \
+  matrix.coeff[0][1] = (ax - bx);   \
+  matrix.coeff[1][1] = (ay - by);  \
+}
+
+#define gegl_unmap(xx,yy,ud,vd) {                                  \
+                  float rx, ry;                                    \
+                  transform.mapfun (&transform, xx, yy, &rx, &ry); \
                   ud = rx;vd = ry;}
-                gegl_sampler_compute_scale (scale_matrix, u, v);
+                gegl_sampler_compute_scale2 (scale_matrix, u, v);
                 gegl_unmap(u,v, cx, cy);
 #undef gegl_unmap
 
-                gegl_sampler_get (sampler,
-                                  cx * in_rect.width, cy * in_rect.height,
-                                  scale, out, GEGL_ABYSS_LOOP);
+#if 1
+                if (scale_matrix.coeff[0][0] > 0.5f)
+                  scale_matrix.coeff[0][0] = (scale_matrix.coeff[0][0]-1.0) * transform.in_width;
+                else if (scale_matrix.coeff[0][0] < -0.5f) 
+                  scale_matrix.coeff[0][0] = (scale_matrix.coeff[0][0]+1.0) * transform.in_width;
+                else
+                  scale_matrix.coeff[0][0] *= transform.in_width;
+
+                if (scale_matrix.coeff[0][1] > 0.5f)
+                  scale_matrix.coeff[0][1] = (scale_matrix.coeff[0][1]-1.0) * transform.in_width;
+                else if (scale_matrix.coeff[0][1] < -0.5f)
+                  scale_matrix.coeff[0][1] = (scale_matrix.coeff[0][1]+1.0) * transform.in_width;
+                else
+                  scale_matrix.coeff[0][1] *= transform.in_width;
+
+                scale_matrix.coeff[1][1] *= transform.in_height;
+                scale_matrix.coeff[1][0] *= transform.in_height;
+#endif
+                getfun (sampler,
+                        cx * transform.in_width + 0.5f,
+                        cy * transform.in_height + 0.5f,
+                        scale, out, abyss_mode);
+
                 out += 4;
 
                 /* update x, y and u,v coordinates */
-                x++;
+                x--;
                 u+=ud;
-                if (x >= (it->roi->x + it->roi->width))
+                if (x == 0)
                   {
-                    x = it->roi->x;
-                    y++;
+                    x = it->roi->width;
                     u = u0;
                     v += vd;
                   }
@@ -448,22 +548,20 @@ process (GeglOperation       *operation,
             for (i=0; i<n_pixels; i++)
               {
                 float cx, cy;
-
-                transform.xy2ll (&transform, u, v, &cx, &cy);
-
-                gegl_sampler_get (sampler,
-                                  cx * in_rect.width, cy * in_rect.height,
-                                  scale, out, GEGL_ABYSS_LOOP);
+                transform.mapfun (&transform, u, v, &cx, &cy);
+                getfun (sampler,
+                        cx * transform.in_width + 0.5f,
+                        cy * transform.in_height + 0.5f,
+                        scale, out, abyss_mode);
                 out += 4;
 
                 /* update x, y and u,v coordinates */
-                x++;
+                x--;
                 u+=ud;
-                if (x >= (it->roi->x + it->roi->width))
+                if (x <= 0)
                   {
-                    x = it->roi->x;
+                    x = it->roi->width;
                     u = u0;
-                    y++;
                     v += vd;
                   }
               }
@@ -472,43 +570,6 @@ process (GeglOperation       *operation,
   }
 
   g_object_unref (sampler);
-
-#if 0
-  {
-    float t;
-    float lat0  = 0;
-    float lon0 = 0;
-    float lat1  = 0.5;
-    float lon1 = 0.5;
-    int i = 0;
-    guchar pixel[4] = {255,0,0,255};
-
-    for (t = 0; t < 1.0; t+=0.01, i++)
-    {
-      float lat = lat0 * (1.0 - t) + lat1 * t;
-      float lon = lon0 * (1.0 - t) + lon1 * t;
-      float x, y;
-      float xx, yy;
-      GeglRectangle prect = {0,0,1,1};
-
-      ll2xy (&transform, lon, lat, &x, &y);
-
-      x += xoffset;
-      y += 0.5;
-
-      x *= width;
-      y *= height;
-
-      prect.x = floor (x);
-      prect.y = floor (y);
-      prect.width = 1;
-      prect.height = 1;
-
-      gegl_buffer_set (output, &prect, 0, babl_format ("R'G'B' u8"), pixel, 8);
-    }
-  }
-#endif
-
   return TRUE;
 }
 
@@ -539,10 +600,10 @@ gegl_op_class_init (GeglOpClass *klass)
     "name",                  "gegl:panorama-projection",
     "title",                 _("Panorama Projection"),
     "reference-composition", composition,
-    "reference-hash",        "216b28fc8471fb8a741d9b42ac328d37",
+    "reference-hash",        "3ab9831053ff0a9e32623ecc8a148e67",
     "position-dependent",    "true",
     "categories" ,           "map",
-    "description", _("Perform an equilinear/gnomonic or little planet/stereographic projection of an equirectangular input image."),
+    "description", _("Do panorama viewer rendering mapping or its inverse for an equirectangular input image. (2:1 ratio containing 360x180 degree panorama)."),
     NULL);
 }
 #endif
